@@ -31,21 +31,36 @@ class WordVectors(object):
         self.vectors = vectors
         self.clusters = clusters
 
+        self.vocab_hash = {}
+        for i, word in enumerate(vocab):
+            self.vocab_hash[word] = i
+
     def ix(self, word):
         """
         Returns the index on self.vocab and `self.vectors` for `word`
         """
-        temp = np.where(self.vocab == word)[0]
-        if temp.size == 0:
-            raise KeyError('Word not in vocabulary')
-        else:
-            return temp[0]
+        return self.vocab_hash[word]
+
+    def word(self, ix):
+        """Returns the word that corresponds to the index.
+
+        Parameters
+        -------
+        ix : int
+            The index of the word
+
+        Returns
+        -------
+        str
+            The word that corresponds to the index
+        """
+        return self.vocab[ix]
 
     def __getitem__(self, word):
         return self.get_vector(word)
 
     def __contains__(self, word):
-        return word in self.vocab
+        return word in self.vocab_hash
 
     def get_vector(self, word):
         """
@@ -54,6 +69,22 @@ class WordVectors(object):
         idx = self.ix(word)
         return self.vectors[idx]
 
+    def get_word(self, vector):
+        """Returns the word according to the vector
+
+        Parameters
+        -------
+        vector : numpy.core.multiarray.array
+            The representing vector of the word
+
+        Returns
+        -------
+        str or None
+            The word according to the specified vector if found, else None
+        """
+        word_index = np.where(np.all(self.vectors == vector, axis=1))[0]
+        return self.word(word_index[0]) if word_index.size else None
+
     def cosine(self, word, n=10):
         """
         Cosine similarity.
@@ -136,7 +167,7 @@ class WordVectors(object):
         joblib.dump(self, fname)
 
     @classmethod
-    def from_binary(cls, fname, vocabUnicodeSize=78, desired_vocab=None):
+    def from_binary(cls, fname, vocabUnicodeSize=78, desired_vocab=None, encoding="utf-8", newLines=True):
         """
         Create a WordVectors class based on a word2vec binary file
 
@@ -160,21 +191,22 @@ class WordVectors(object):
             binary_len = np.dtype(np.float32).itemsize * vector_size
             for i in range(vocab_size):
                 # read word
-                word = ''
+                word = b''
                 while True:
-                    ch = fin.read(1).decode('ISO-8859-1')
-                    if ch == ' ':
+                    ch = fin.read(1)
+                    if ch == b' ':
                         break
                     word += ch
                 include = desired_vocab is None or word in desired_vocab
                 if include:
-                    vocab[i] = word
+                    vocab[i] = word.decode(encoding)
 
                 # read vector
                 vector = np.fromstring(fin.read(binary_len), dtype=np.float32)
                 if include:
                     vectors[i] = unitvec(vector)
-                fin.read(1)  # newline
+                if newLines:
+                    fin.read(1)  # newline
 
             if desired_vocab is not None:
                 vectors = vectors[vocab != '', :]
@@ -182,7 +214,7 @@ class WordVectors(object):
         return cls(vocab=vocab, vectors=vectors)
 
     @classmethod
-    def from_text(cls, fname, vocabUnicodeSize=78, desired_vocab=None):
+    def from_text(cls, fname, vocabUnicodeSize=78, desired_vocab=None, encoding="utf-8"):
         """
         Create a WordVectors class based on a word2vec text file
 
@@ -204,7 +236,7 @@ class WordVectors(object):
             vocab = np.empty(vocab_size, dtype='<U%s' % vocabUnicodeSize)
             vectors = np.empty((vocab_size, vector_size), dtype=np.float)
             for i, line in enumerate(fin):
-                line = line.decode('ISO-8859-1').strip()
+                line = line.decode(encoding).strip()
                 parts = line.split(' ')
                 word = parts[0]
                 include = desired_vocab is None or word in desired_vocab
